package controler.impl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import beans.Asistencia;
import beans.Curso;
import beans.Matricula;
import beans.Persona;
import beans.Pista;
import controler.CursoControler;
import dao.AsistenciaDAO;
import dao.CursoDAO;
import dao.MatriculaDAO;
import dao.PersonaDAO;
import dao.PistaDAO;

public class CursoControlerImpl implements CursoControler{
	
	private CursoDAO cursoDAO;
	private MatriculaDAO matriculaDAO;
	private AsistenciaDAO asistenciaDAO;
	private PersonaDAO personaDAO;
	private PistaDAO pistaDAO;
	

	public void setCursoDAO(CursoDAO cursoDAO) {
		this.cursoDAO = cursoDAO;
	}
	
	public void setMatriculaDAO(MatriculaDAO matriculaDAO) {
		this.matriculaDAO = matriculaDAO;
	}
	
	public void setAsistenciaDAO(AsistenciaDAO asistenciaDAO) {
		this.asistenciaDAO = asistenciaDAO;
	}

	public void setPersonaDAO(PersonaDAO personaDAO) {
		this.personaDAO = personaDAO;
	}
	
	public void setPistaDAO(PistaDAO pistaDAO){
		this.pistaDAO = pistaDAO;
	}
	
	public Curso getCurso(String idCurso) throws Exception{
		return cursoDAO.getCurso(idCurso);
	}
	
	public List<Curso> getCursos() throws Exception{
		return cursoDAO.getCursos();
	}
	
	public void borrarCurso(Curso curso) throws Exception{
		cursoDAO.borrarCurso(curso);
	}

	@Override
	public List<Curso> getCursosLibres(Persona alumno) throws Exception {

		List<Curso> cursos = cursoDAO.getCursosLibres();
		List<Curso> eliminables = new ArrayList<Curso>();
 		List<Matricula> matriculas = matriculaDAO.getMatriculasAlumno(alumno);
		
		for(Matricula mat:matriculas){
			Curso curs = mat.getCurso();
			for(Curso curso:cursos){
				if(curso.getIdCurso().equals(curs.getIdCurso()))
					eliminables.add(curso);
				}
		}
		for(Curso curso:eliminables){
			cursos.remove(curso);
		}
		
		return cursos;
	}
	
	public void guardarCurso(Curso curso) throws Exception{
		cursoDAO.guardarCurso(curso);
	}
	
	@Override
	public void inscribirAlumnoEnCurso(Persona alumno, String idCurso)
			throws Exception {
		Curso curso = cursoDAO.getCurso(idCurso);
		matriculaDAO.inscribirAlumnoEnCurso(alumno, curso);
		curso.setPlazasDisponibles(curso.getPlazasDisponibles()-1);
		cursoDAO.guardarCurso(curso);
	}
	
	public void borrarAlumnoDeCurso(String idMatricula)
			throws Exception {
		Matricula matricula = matriculaDAO.getMatricula(idMatricula);
		matriculaDAO.borrarAlumnoDeCurso(matricula);
	}

	@Override
	public List<Matricula> getMatriculasAlumno(Persona alumno) throws Exception {
		return matriculaDAO.getMatriculasAlumno(alumno);
	}

	public List<Curso> getCursosProfesor(Persona profesor) throws Exception{
		return cursoDAO.getCursosProfesor(profesor);
	}
	
	public List<Matricula> getMatriculasCurso(String idCurso)
			throws Exception {
		Curso curso = cursoDAO.getCurso(idCurso);
		return matriculaDAO.getMatriculasCurso(curso);
	}
	
	public Date ultimaFechaCurso(String idCurso) throws Exception {
		Curso curso = cursoDAO.getCurso(idCurso);
        GregorianCalendar cal = (GregorianCalendar) GregorianCalendar.getInstance();
        //cal.add(Calendar.DAY_OF_WEEK, 3);
        Boolean [] dias = new Boolean[5];

        dias[0] = curso.getLunes();
        dias[1] = curso.getMartes();
        dias[2] = curso.getMiercoles();
        dias[3] = curso.getJueves();
        dias[4] = curso.getViernes();
        System.out.println(cal.getTime());

        int diaInicial = cal.get(Calendar.DAY_OF_WEEK) - 1;

        if(diaInicial==0)
                diaInicial=7;
        int index = 0;
        for(int i= 0;i<dias.length;i++){

                if(dias[i]==true){
                        index = i+1;
                        if(index==diaInicial)
                                break;
                }
        }
        cal.add(Calendar.DAY_OF_WEEK, index-diaInicial);

        if((diaInicial-index)<0)
                cal.add(Calendar.DAY_OF_MONTH, -7);

        System.out.println(cal.getTime());
        return cal.getTime();
}
	
	public Boolean existeAsistencia(String idCurso, Date fechaAsistencia) throws Exception{
		Curso curso = cursoDAO.getCurso(idCurso);
		return asistenciaDAO.existeAsistencia(curso, fechaAsistencia);
	}
	
	public void marcarAsistencia(String idAlumno, String idCurso, Date fechaAsistencia)
			throws Exception{
		Curso curso = cursoDAO.getCurso(idCurso);
		Persona alumno = personaDAO.getPersona(idAlumno);
		asistenciaDAO.marcarAsistencia(alumno, curso, fechaAsistencia);
	}
	
	public List<Pista> getPistas() throws Exception {		
		return pistaDAO.getPistas();
	}
	
	public List<Persona> getProfesores() throws Exception{
		return personaDAO.getProfesores();
	}
	
	public List<Asistencia> getAsistenciaCurso(String idCurso) throws Exception{
		Curso curso = cursoDAO.getCurso(idCurso);
		Date fechaAsistencia = asistenciaDAO.getUltimaFechaAsistencia(curso);
		return asistenciaDAO.getAsistencia(curso, fechaAsistencia);
	}
}
