package dao.impl;

import java.sql.Time;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;

import beans.Alquiler;
import beans.Persona;
import dao.AlquilerDAO;
import dao.BaseDAO;

public class AlquilerDAOImpl extends BaseDAO implements AlquilerDAO{
	
	@SuppressWarnings("unchecked")
	public List<Alquiler> getAlquileresPersona(Persona alquilador)
			throws Exception {

		Session session = null;
		Transaction tx = null;
		List<Alquiler> alquileres = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Alquiler.class);

		   crit.add(Expression.eq("alquilador.idPersona", alquilador.getIdPersona()));
		   crit.add(Expression.ge("fechaAlquiler", new Date()));
		   alquileres = crit.list();

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return alquileres;
	}
	
	public Boolean pistaOcupada(Date fecha, Time horaIni, Time horaFin, String idPista) throws Exception{
		Session session = null;
		Transaction tx = null;
		Boolean devolver = false;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();
		   SimpleDateFormat formatoDelTexto = new SimpleDateFormat("dd/MM/yyyy");

		   SQLQuery query = 
				   session.createSQLQuery("select 1 from alquiler " +
				   		"where idPista="+idPista+" and fechaAlquiler='"+formatoDelTexto.format(fecha)+"' " +
				   		"and (" +
				   		"('"+horaIni+"' Between horaIniAlquiler and horaFinAlquiler) " +
				   		"or (horaIniAlquiler Between '"+horaIni+"' and '"+horaFin+"') " +
				   		"or (horaFinAlquiler Between '"+horaIni+"' and '"+horaFin+"') " +
				   		"or ('"+horaFin+"' Between horaIniAlquiler and horaFinAlquiler))");
		   if(!query.list().isEmpty())
			   devolver = true;

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return devolver;
	}
	
	public void alquilarPista(Alquiler alq)
			throws Exception {
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.save(alq);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	public Alquiler getAlquiler(String idAlquiler) throws Exception{
		Session session = null;
		Transaction tx = null;
		Alquiler alquiler = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Alquiler.class);
		   crit.add(Expression.eq("idAlquiler",Integer.parseInt(idAlquiler)));
		   alquiler = (Alquiler) crit.list().get(0);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return alquiler;
	}
	
	public void borrarAlquiler(Alquiler alquiler)
			throws Exception {
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.delete(alquiler);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
}
