package dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.Expression;
import org.hibernate.criterion.Order;

import beans.Asistencia;
import beans.Curso;
import beans.Persona;
import dao.AsistenciaDAO;
import dao.BaseDAO;

public class AsistenciaDAOImpl extends BaseDAO implements AsistenciaDAO{

	public void marcarAsistencia(Persona alumno, Curso curso, Date fechaAsistencia)
			throws Exception {
		Asistencia asist = new Asistencia();
		asist.setAlumno(alumno);
		asist.setCurso(curso);
		asist.setFechaAsistencia(fechaAsistencia);
		Session session = null;
		Transaction tx = null;


		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.save(asist);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	public Boolean existeAsistencia(Curso curso, Date fechaAsistencia)
			throws Exception {
		Asistencia asist = new Asistencia();
		asist.setCurso(curso);
		asist.setFechaAsistencia(fechaAsistencia);
		Session session = null;
		Transaction tx = null;
		Boolean devolver = true;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Asistencia.class);
		   crit.add(Example.create(asist));
		   devolver = !crit.list().isEmpty();

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return devolver;
	}
	
	@SuppressWarnings("unchecked")
	public List<Asistencia> getAsistencia(Curso curso, Date fechaAsistencia)
			throws Exception {
		Asistencia asist = new Asistencia();
		asist.setCurso(curso);
		asist.setFechaAsistencia(fechaAsistencia);
		Session session = null;
		Transaction tx = null;
		List<Asistencia> devolver = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Asistencia.class);
		   crit.add(Example.create(asist));
		   devolver = crit.list();

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return devolver;
	}
	
	@SuppressWarnings("unchecked")
	public Date getUltimaFechaAsistencia(Curso curso)
			throws Exception {
		Asistencia asist;
		Session session = null;
		Transaction tx = null;
		Date devolver = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Asistencia.class);
		   crit.add(Expression.eq("curso.idCurso",curso.getIdCurso()));
		   crit.addOrder(Order.desc("fechaAsistencia"));
		   
		   List<Asistencia> aux = crit.list();
		   if(!aux.isEmpty()){
			   asist = (Asistencia) crit.list().get(0);
			   devolver = asist.getFechaAsistencia();
		   }else{
			   devolver = new Date();
		   }
		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return devolver;
	}

}
