package dao.impl;

import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;

import beans.Curso;
import beans.Persona;
import dao.BaseDAO;
import dao.CursoDAO;

public class CursoDAOImpl extends BaseDAO implements CursoDAO{
	
	@SuppressWarnings("unchecked")
	public List<Curso> getCursos() throws Exception {

		Session session = null;
		Transaction tx = null;
		List<Curso> cursos;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Curso.class);
		   cursos = crit.list();

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return cursos;
	}

	@SuppressWarnings("unchecked")
	public List<Curso> getCursosLibres() throws Exception {


		Session session = null;
		Transaction tx = null;
		List<Curso> cursos;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Curso.class);
		   crit.add(Expression.ne("plazasDisponibles",0));
		   crit.add(Expression.ge("fechaFin", new Date()));
		   crit.add(Expression.le("fechaInicio", new Date()));
		   cursos = crit.list();


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return cursos;
	}
	
	public void guardarCurso(Curso curso) throws Exception {
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.saveOrUpdate(curso);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	public void borrarCurso(Curso curso) throws Exception {
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.delete(curso);


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	public Curso getCurso(String idCurso) throws Exception{
		Session session = null;
		Transaction tx = null;
		Curso curso = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Curso.class);
		   crit.add(Expression.eq("idCurso",Integer.parseInt(idCurso)));
		   curso = (Curso) crit.list().get(0);


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return curso;
	}
	
	@SuppressWarnings("unchecked")
	public List<Curso> getCursosProfesor(Persona profesor) throws Exception {


		Session session = null;
		Transaction tx = null;
		List<Curso> clases;
		Curso clase = new Curso();
		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();
		   clase.setProfesor(profesor);
		   Criteria crit = session.createCriteria(Curso.class);
		   crit.add(Expression.eq("profesor.idPersona",profesor.getIdPersona()));
		   clases = crit.list();


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return clases;
	}

}
