package dao.impl;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.hibernate.criterion.Expression;

import beans.Curso;
import beans.Matricula;
import beans.Persona;
import dao.BaseDAO;
import dao.MatriculaDAO;

public class MatriculaDAOImpl extends BaseDAO implements MatriculaDAO{

	@Override
	public void inscribirAlumnoEnCurso(Persona alumno, Curso curso)
			throws Exception {
		Matricula mat = new Matricula();
		mat.setAlumno(alumno);
		mat.setCurso(curso);
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.save(mat);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	public void borrarAlumnoDeCurso(Matricula matricula)
			throws Exception {
		Session session = null;
		Transaction tx = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   session.delete(matricula);

		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
	}
	
	@SuppressWarnings("unchecked")
	public List<Matricula> getMatriculasAlumno(Persona alumno)
			throws Exception {
		Session session = null;
		Transaction tx = null;
		List<Matricula> matriculas = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Matricula.class);
		   crit.add(Expression.eq("alumno.idPersona",alumno.getIdPersona()));
		   matriculas = crit.list();


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return matriculas;
	}
	
	@SuppressWarnings("unchecked")
	public List<Matricula> getMatriculasCurso(Curso curso)
			throws Exception {

		Session session = null;
		Transaction tx = null;
		List<Matricula> matriculas = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Matricula.class);
		   crit.add(Expression.eq("curso.idCurso",curso.getIdCurso()));
		   matriculas = crit.list();


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return matriculas;
	}

	@Override
	public Matricula getMatricula(String idMatricula) throws Exception {
		Session session = null;
		Transaction tx = null;
		Matricula matricula = null;

		try {
		   session = sessionFactory.openSession();
		   tx = session.beginTransaction();

		   Criteria crit = session.createCriteria(Matricula.class);
		   crit.add(Expression.eq("idMatricula",Integer.parseInt(idMatricula)));
		   matricula = (Matricula) crit.list().get(0);


		   tx.commit();
		}
		catch(Exception e) {
		   if (tx != null)    {
		      try  {
		         tx.rollback();
		      }
		      catch (HibernateException e1)  {
		    	  log.error(e1);
		         throw new Exception(e1);
		      }
		   }
		   log.error(e);
		   throw new Exception(e);
		}
		finally {
		   if (session != null) {
		      try {
		         session.close();
		      }
		      catch (HibernateException e)  {
		    	  log.error(e);
		      }
		   }
		}
		return matricula;
	}

}
